unit uOptions;
{$INCLUDE 'VirusTotal.inc'}

interface

uses
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  SysUtils;

type
  TOptions = class(TFarConfig)
  private
    fApiKey: string;
    fMaxFileSize: int64;
    fMaxFileSizeStr: string;
    fInitialScanTime: integer;
    fDelayBetweenReportChecks: integer;
    fCommandLinePrefix: string;
    fUseRescanAPI: boolean;
    function GetMaxFileSize: int64;
    procedure SetMaxFileSizeStr(const Value: string);
  protected
  public
    constructor Create;
    destructor Destroy; override;
    procedure Init;
    procedure Load;
    procedure Save;
    procedure Assign(Options: TOptions);
    property ApiKey: string read fApiKey write fApiKey;
    property MaxFileSizeStr: string read fMaxFileSizeStr write SetMaxFileSizeStr;
    property MaxFileSize: int64 read GetMaxFileSize;
    property InitialScanTime: integer read fInitialScanTime write fInitialScanTime;
    property DelayBetweenReportChecks: integer read fDelayBetweenReportChecks write fDelayBetweenReportChecks;
    property CommandLinePrefix: string read fCommandLinePrefix write fCommandLinePrefix;
    property UseRescanAPI: boolean read fUseRescanAPI write fUseRescanAPI;
  end;

implementation

uses
  uMessages;

const
  MAX_FILE_SIZE = 32*1024*1024;
  MAX_FILE_SIZE_STR = '32M';

{ TDirSyncOptions }

procedure TOptions.Assign(Options: TOptions);
begin
  ApiKey := Options.ApiKey;
  MaxFileSizeStr := Options.MaxFileSizeStr;
  InitialScanTime := Options.InitialScanTime;
  DelayBetweenReportChecks := Options.DelayBetweenReportChecks;
end;

constructor TOptions.Create;
begin
  inherited Create('VirusTotal', False);
  Init;
end;

destructor TOptions.Destroy;
begin
  inherited;
end;

function TOptions.GetMaxFileSize: int64;
type
  TState = (stLeadingSpace, stDigits, stSpace, stMultiplier, stTrailingSpace);
var
  State: TState;
  n: int64;
  i: integer;
begin
  if fMaxFileSize < 0 then
  begin
    n := 0;
    State := stLeadingSpace;
    for i := 1 to Length(fMaxFileSizeStr) do
    begin
      case fMaxFileSizeStr[i] of
        '0'..'9':
          begin
            if State = stLeadingSpace then
              State := stDigits;
            if State <> stDigits then
            begin
              ShowWarning(Format(GetMsg(MInvalidMaxFileSize1), [i]));
              Result := MAX_FILE_SIZE;
              Exit;
            end;
            n := 10*n + Ord(fMaxFileSizeStr[i]) - Ord('0');
          end;
        ' ', #9:
          begin
            if State = stDigits then
              State := stSpace;
            if State = stMultiplier then
              State := stTrailingSpace;
          end;
        'K', 'k':
          begin
            if not (State in [stDigits, stSpace]) then
            begin
              ShowWarning(Format(GetMsg(MInvalidMaxFileSize2), [i]));
              Result := MAX_FILE_SIZE;
              Exit;
            end;
            State := stTrailingSpace;
            n := n * 1024;
          end;
        'M', 'm':
          begin
            if not (State in [stDigits, stSpace]) then
            begin
              ShowWarning(Format(GetMsg(MInvalidMaxFileSize2), [i]));
              Result := MAX_FILE_SIZE;
              Exit;
            end;
            State := stTrailingSpace;
            n := n * 1024*1024;
          end;
        'G', 'g':
          begin
            if not (State in [stDigits, stSpace]) then
            begin
              ShowWarning(Format(GetMsg(MInvalidMaxFileSize2), [i]));
              Result := MAX_FILE_SIZE;
              Exit;
            end;
            State := stTrailingSpace;
            n := n * 1024*1024*1024;
          end;
        else
        begin
          ShowWarning(Format(GetMsg(MInvalidMaxFileSize3), [i]));
          Result := MAX_FILE_SIZE;
          Exit;
        end;
      end;
    end;
    fMaxFileSize := n;
  end;
  Result := fMaxFileSize;
end;

procedure TOptions.Init;
begin
  fApiKey := '';
  fMaxFileSize := -1;
  fMaxFileSizeStr := MAX_FILE_SIZE_STR;
  fInitialScanTime := 60;
  fDelayBetweenReportChecks := 20;
  fCommandLinePrefix := 'vt';
  fUseRescanAPI := False;
end;

procedure TOptions.Load;
begin
  if Open then
    try
      ApiKey := ReadString('ApiKey', ApiKey);
      MaxFileSizeStr := ReadString('MaxFileSizeStr', MaxFileSizeStr);
      InitialScanTime := ReadInteger('InitialScanTime', InitialScanTime);
      DelayBetweenReportChecks := ReadInteger('DelayBetweenReportChecks', DelayBetweenReportChecks);
      CommandLinePrefix := ReadString('CommandLinePrefix', CommandLinePrefix);
      UseRescanAPI := ReadBoolean('UseRescanAPI', UseRescanAPI);
    finally
      Close;
    end;
end;

procedure TOptions.Save;
begin
  if Open then
    try
      WriteString('ApiKey', ApiKey);
      WriteString('MaxFileSizeStr', MaxFileSizeStr);
      WriteInteger('InitialScanTime', InitialScanTime);
      WriteInteger('DelayBetweenReportChecks', DelayBetweenReportChecks);
      WriteString('CommandLinePrefix', CommandLinePrefix);
      WriteBoolean('UseRescanAPI', UseRescanAPI);
    finally
      Close;
    end;
end;

procedure TOptions.SetMaxFileSizeStr(const Value: string);
begin
  fMaxFileSizeStr := Value;
  fMaxFileSize := -1;
end;

end.
